#include "cameracontroler.h"
#include "cameratrainer.h"
#include "camerarecognition.h"
#include <QFileInfo>

using namespace cv;
using namespace std;

CameraControler::CameraControler(){
    cap.open(0);
    estadoCamara = false;
    if(!detectorFace.load("./haarcascades/haarcascade_frontalface_alt.xml"))
        cout << "Error detector de rostro" << endl;
    if(!detectorEye.load("./haarcascades/haarcascade_eye_tree_eyeglasses.xml"))
        cout << "Error detector de ojos" << endl;
    EYE_SX = 0.16f;
    EYE_SY = 0.26f;
    EYE_SW = 0.30f;
    EYE_SH = 0.28f;
    DESIRED_LEFT_EYE_Y = 0.14;
    DESIRED_LEFT_EYE_X = 0.19;
    FaceWidth = 100;
    FaceHeight = 100;
    faceRecognizer = cv::face::LBPHFaceRecognizer::create();
    QFileInfo check("Modelo.txt");
    if(check.exists()){
        faceRecognizer->read("Modelo.txt");
    }
    IsUpdated = false;
}

CameraControler::~CameraControler(){
    cap.release();
}

void CameraControler::setCameraTrainer(CameraTrainer *cameraTrainer){
    window = cameraTrainer;
}

void CameraControler::setEstadoCamara(bool estadoCamara){
    this->estadoCamara = estadoCamara;
}

void CameraControler::setCameraRecognition(CameraRecognition *cameraRecognition){
    this->cameraRecognition = cameraRecognition;
}

void CameraControler::setIamTraining(bool IamTraining){
    this->IamTraining = IamTraining;
}

void CameraControler::update(){
    IsUpdated = true;
}

void CameraControler::run(){
    if(IamTraining){
        Training();
    }else{
        Recognizing();
    }
}

void CameraControler::Training(){
    Mat Imagen, Imagen2, Gray, ProcessImage, Face, topLeftOfCara,topRightOfCara, FaceCroppedOne, FaceCroppedTwo;
    vector<Rect> Faces, leftEye, rightEye;
    QPixmap pixImage;
    int leftX;
    int topY;
    int widthX;
    int heightY;
    int rightX;
    vector<Mat> FacesReady;
    vector<int> Rol;
    while(estadoCamara){
        cap >> Imagen;
        float escala = Imagen.cols / 480.0;
        int escalaAltura = cvRound(Imagen.rows/escala);
        resize(Imagen,Imagen,Size(480,escalaAltura));
        cvtColor(Imagen,Imagen,COLOR_BGR2RGB);
        resize(Imagen,Imagen2,Size((480*3)/2,(escalaAltura*3)/2));
        pixImage = QPixmap::fromImage(QImage((unsigned char*) Imagen2.data, Imagen2.cols, Imagen2.rows, QImage::Format_RGB888));
        window->setImage(&pixImage);

        cvtColor(Imagen, Gray, CV_BGR2GRAY);
        equalizeHist(Gray, ProcessImage);
        detectorFace.detectMultiScale(ProcessImage, Faces);

        if(Faces.size() == 1){
            Face.release();
            Face = ProcessImage(Faces[0]).clone();
            leftX = cvRound(Face.cols * EYE_SX);
            topY = cvRound(Face.rows * EYE_SY);
            widthX = cvRound(Face.cols * EYE_SW);
            heightY = cvRound(Face.rows * EYE_SH);
            rightX = cvRound(Face.cols * (1.0-EYE_SX-EYE_SW));
            topLeftOfCara = Face(Rect(leftX, topY, widthX, heightY));
            topRightOfCara = Face(Rect(rightX, topY, widthX, heightY));
            detectorEye.detectMultiScale(topLeftOfCara, leftEye);
            detectorEye.detectMultiScale(topRightOfCara, rightEye);
            if(((int)leftEye.size() > 0) && ((int)rightEye.size() > 0)){
                window->setIcanTakePhoto(true);
                if(IsUpdated){
                    Rect left = leftEye[0];
                    Rect right = rightEye[0];

                    left.x += leftX;
                    left.y += topY;

                    right.x += rightX;
                    right.y += topY;

                    this->CropFace(ProcessImage(Faces[0]),FaceCroppedOne,left,right);
                    flip(FaceCroppedOne, FaceCroppedTwo, 1);
                    FacesReady.push_back(FaceCroppedOne);
                    FacesReady.push_back(FaceCroppedTwo);
                    Rol.push_back(window->getRol());
                    Rol.push_back(window->getRol());
                    faceRecognizer->update(FacesReady,Rol);
                    faceRecognizer->write("Modelo.txt");
                    IsUpdated = false;
                    FacesReady.clear();
                    Rol.clear();
                }
            }else{
                window->setIcanTakePhoto(false);
            }
        }else{
            window->setIcanTakePhoto(false);
        }

        QThread::msleep(20);
    }
}

void CameraControler::Recognizing(){
    Mat Imagen, Gray, ProcessImage, Face, topLeftOfCara,topRightOfCara, FaceCropped;
    Mat Imagen2;
    vector<Rect> Faces, leftEye, rightEye;
    QPixmap pixImage;
    int leftX, topY, widthX, heightY, rightX;
    int i = 0;
    double Accuracy = 0.0;
    int Rol = -1;
    while(estadoCamara){
        cap >> Imagen;
        float escala = ((float) Imagen.cols) / ((float) 480);
        int escalaAltura = cvRound(Imagen.rows/escala);
        resize(Imagen,Imagen,Size(480,escalaAltura));
        cvtColor(Imagen,Imagen,COLOR_BGR2RGB);
        resize(Imagen,Imagen2,Size((480*3)/2,(escalaAltura*3)/2));
        pixImage = QPixmap::fromImage(QImage((unsigned char*) Imagen2.data, Imagen2.cols, Imagen2.rows, QImage::Format_RGB888));
        cameraRecognition->setImage(&pixImage,0);

        cvtColor(Imagen, Gray, CV_BGR2GRAY);
        equalizeHist(Gray, ProcessImage);
        detectorFace.detectMultiScale(ProcessImage, Faces);

        for(i = 0; i < (int) Faces.size(); i++){
            Face.release();
            Face = ProcessImage(Faces[i]).clone();
            leftX = cvRound(Face.cols * EYE_SX);
            topY = cvRound(Face.rows * EYE_SY);
            widthX = cvRound(Face.cols * EYE_SW);
            heightY = cvRound(Face.rows * EYE_SH);
            rightX = cvRound(Face.cols * (1.0-EYE_SX-EYE_SW));
            topLeftOfCara = Face(Rect(leftX, topY, widthX, heightY));
            topRightOfCara = Face(Rect(rightX, topY, widthX, heightY));
            detectorEye.detectMultiScale(topLeftOfCara, leftEye);
            detectorEye.detectMultiScale(topRightOfCara, rightEye);
            if(((int)leftEye.size() > 0) && ((int)rightEye.size() > 0)){
                Rect left = leftEye[0];
                Rect right = rightEye[0];
                left.x += leftX;
                left.y += topY;
                right.x += rightX;
                right.y += topY;
                this->CropFace(ProcessImage(Faces[i]),FaceCropped,left,right);
                faceRecognizer->setThreshold(80);
                faceRecognizer->predict(FaceCropped,Rol,Accuracy);    
                if(Rol>=0){
                    rectangle(Imagen,
                              Point(Faces[i].x, Faces[i].y),
                              Point(Faces[i].x + Faces[i].width, Faces[i].y + Faces[i].height),
                              CV_RGB(128,202,111), 2);
                    putText(Imagen,to_string(Rol),
                            Point(Faces[i].x,Faces[i].y+Faces[i].height+20),
                            FONT_HERSHEY_DUPLEX,
                            0.5,
                            CV_RGB(51,255,57),
                            1,
                            CV_AA);
                    resize(Imagen,Imagen2,Size((480*3)/2,(escalaAltura*3)/2));
                    pixImage = QPixmap::fromImage(QImage((unsigned char*) Imagen2.data, Imagen2.cols, Imagen2.rows, QImage::Format_RGB888));
                    cameraRecognition->setImage(&pixImage,1);
                }
            }
        }

        QThread::msleep(20);
    }
}

VideoCapture *CameraControler::getCap(){
    return &cap;
}


void CameraControler::CropFace(const Mat& face, Mat& warped, Rect leftEye, Rect rightEye){

    Point left = Point(leftEye.x + leftEye.width/2, leftEye.y + leftEye.height/2);
    Point right = Point(rightEye.x + rightEye.width/2, rightEye.y + rightEye.height/2);
    Point2f eyesCenter = Point2f( (left.x + right.x) * 0.5f, (left.y + right.y) * 0.5f );

    double dy = (right.y - left.y);
    double dx = (right.x - left.x);
    double len = sqrt(dx*dx + dy*dy);
    double angle = atan2(dy, dx) * 180.0 / CV_PI;

    const double DESIRED_RIGHT_EYE_X = (1.0f - DESIRED_LEFT_EYE_X);

    double desiredLen = (DESIRED_RIGHT_EYE_X - DESIRED_LEFT_EYE_X) * FaceWidth;
    double scale = desiredLen / len;

    Mat rot_mat = getRotationMatrix2D(eyesCenter, angle, scale);

    rot_mat.at<double>(0, 2) += FaceWidth * 0.5f - eyesCenter.x;
    rot_mat.at<double>(1, 2) += FaceHeight * DESIRED_LEFT_EYE_Y - eyesCenter.y;

    warped = Mat(FaceHeight, FaceWidth, CV_8U, Scalar(128));

    warpAffine(face, warped, rot_mat, warped.size());

}
