#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "registrocontacto.h"
#include "camerarecognition.h"
#include "cameracontroler.h"

namespace Ui {
class MainWindow;
}

/**
 * Clase que implementa el comportamiento de MainWindow
 * Es la clase de la ventana principal y contiene todos los elementos del programa.
 * @author Luis Leiva, Felipe Muñoz, Sebastian Ubiergo
 */
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    /**
     * Constructor de MainWindow. Inicializa un nuevo objeto.
     * @param parent:
     */
    explicit MainWindow(QWidget *parent = 0);
    /**
     * Destructor de MainWindow.
     */
    ~MainWindow();

public slots:
    /**
     * Muestra la ventana de Registro Contacto y setea el controlador de la camara para que funcione en modo entrenamiento.
     */
    void CreateWindowRegistro(void);
    /**
     * Muestra la ventana de Camara Recognition y setea el controlador de la camara para que funcione en modo reconocimiento.
     */
    void CreateWindowRecognition(void);

private:
    Ui::MainWindow *ui;
    RegistroContacto *registro;
    CameraRecognition *cameraRecognition;
    CameraControler *cameraControler;
};

#endif // MAINWINDOW_H
