#ifndef REGISTROCONTACTO_H
#define REGISTROCONTACTO_H

#include <QDialog>
#include "cameratrainer.h"
#include "cameracontroler.h"
#include "errorwindow.h"

namespace Ui {
class RegistroContacto;
}

/**
 * Clase que implementa el comportamiento de RegistroContacto.
 * Es la clase de la ventana para registrar los datos de la persona antes de hacer la toma de datos.
 * @author Luis Leiva, Felipe Muñoz, Sebastian Ubiergo
 */
class RegistroContacto : public QDialog
{
    Q_OBJECT

public:
    /**
     * Constructor de RegistroContacto. Inicializa un nuevo objeto.
     * @param cameraControler: Es un puntero del objeto que controla la camara.
     * @param parent:
     */
    explicit RegistroContacto(CameraControler *cameraControler,QWidget *parent = 0);
    /**
     * Destructor de RegistroContacto
     */
    ~RegistroContacto();
    /**
     * Limpiar los campos rellenables por el usuario.
     */
    void clearAll(void);

public slots:
    /**
     * Abre la ventana de cameraTrainer para hacer la recopilacion de datos.
     */
    void CreateWindowCaption(void);
    /**
     * Cierra la ventana de cameraTrainer y se abre está ventana-
     */
    void OpenWindows(void);

private:
    Ui::RegistroContacto *ui;
    CameraTrainer *cameraTrainer;
    ErrorWindow *errorWindow;

};

#endif // REGISTROCONTACTO_H
